--- Guilder: Crowd Control Module
-- This module allows the user to set the crowd control responsibilities and report them to various places
-- @class file
-- @name Modules/Crowd.lua
-- @release 1.1.0

if select(6, GetAddOnInfo("Guilder_" .. (debugstack():match("[o%.][d%.][u%.]les\\(.-)\\") or ""))) ~= "MISSING" then return end

local Guilder = _G.Guilder
if not Guilder then
  error("Guilder_Crowd requires Guilder")
end
local L = Guilder.L

local DEBUG = Guilder.DEBUG
local expect = Guilder.expect

-- default settings for the addon -------------------------------------------

local DATABASE_DEFAULTS = {
	["channel"] = "SELF",
}
do
	for i = 1, 8 do
		DATABASE_DEFAULTS["RT"..i.."name"] = ""
		DATABASE_DEFAULTS["RT"..i.."type"] = "None"
	end
end

-- declare the module -------------------------------------------------------

local Guilder_Crowd = Guilder:NewModule("Crowd", "AceEvent-3.0", "AceHook-3.0")
Guilder_Crowd:SetName(L["Crowd Control"])
Guilder_Crowd:SetDescription(L["Assign crowd control responsibilities for the current party or raid."])
Guilder_Crowd:SetDefaults(DATABASE_DEFAULTS)
if DEBUG then
	Guilder_Crowd:SetIcon([[Interface\AddOns\Guilder\Modules\Crowd\Crowd]])
else
	Guilder_Crowd:SetIcon([[Interface\AddOns\Guilder_Crowd\Crowd]])
end

-- variables ----------------------------------------------------------------

local targetsLoaded = false
local cc_symbols = {
	[1] = L["Star"],
	[2] = L["Circle"],
	[3] = L["Diamond"],
	[4] = L["Triangle"],
	[5] = L["Moon"],
	[6] = L["Square"],
	[7] = L["Cross"],
	[8] = L["Skull"]
}
local cc_types = {
	["Banish"] = L["Banish"], ["Cyclone"] = L["Cyclone"], ["Enslave"] = L["Enslave"],
	["Fear"] = L["Fear"], ["Hex"] = L["Hex"], ["Hibernate"] = L["Hibernate"],
	["Kite"] = L["Kite"], ["MT"] = L["MT"], ["MindControl"] = L["Mind Control"],
	["None"] = L["None"], ["OT"] = L["OT"], ["Root"] = L["Root"],
	["Sap"] = L["Sap"], ["Tank"] = L["Tank"], ["Seduce"] = L["Seduce"],
	["Shackle"] = L["Shackle"], ["Sheep"] = L["Sheep"], ["Sleep"] = L["Sleep"],
	["Trap"] = L["Trap"], ["TurnEvil"] = L["Turn Evil"]
}

-- Standard Module Methods --------------------------------------------------

-- Is called when the module is loaded
-- @usage internal use only
function Guilder_Crowd:OnModuleLoaded(module)
	if not self.db.profile.enabled then return end
	local id = module.id
end

-- Is called when the module is enabled
-- @usage internal use only
function Guilder_Crowd:OnEnable()
	-- register group/raid changes
	self:RegisterEvent("RAID_ROSTER_UPDATE")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
end

-- Is called when the module is disabled
-- @usage internal use only
function Guilder_Crowd:OnDisable()
	self:UnregisterAllEvents()
end

-- Is called when there is a change to the raid group
-- @usage internal use only
function Guilder_Crowd:RAID_ROSTER_UPDATE()
	if Guilder.configLoaded then
		for i = 1, 8 do
			self.options.responsibilities.args["RT"..i.."name"].values = Guilder:PlayersInGroup()
		end
		LibStub("AceConfigRegistry-3.0"):NotifyChange("Guilder")
	end
end

-- code reuse
Guilder_Crowd.PARTY_MEMBERS_CHANGED = Guilder_Crowd.RAID_ROSTER_UPDATE

-- Is called every time a user opens the configuration screen
-- @usage internal use only
function Guilder_Crowd:UpdateConfigOptions()
	if not targetsLoaded then
		for i = 1, 8 do
			self.options.responsibilities.args["RT"..i.."name"] = {
				type = "select",
				name = cc_symbols[i],
				desc =L["Player responsible for %s."]:format(cc_symbols[i]),
				order = (9 - i) * 10,
				values = Guilder:PlayersInGroup(),
				get = function() return self.db.profile["RT"..i.."name"] end,
				set = function(info, v) self.db.profile["RT"..i.."name"] = v end
			}
			self.options.responsibilities.args["RT"..i.."type"] = {
				type = "select",
				name = L["Crowd Control"],
				desc =L["Crowd control that player should use."],
				order = (9 - i) * 10 + 1,
				values = cc_types,
				get = function() return self.db.profile["RT"..i.."type"] end,
				set = function(info, v) self.db.profile["RT"..i.."type"] = v end
			}
			self.options.responsibilities.args["RT"..i.."clear"] = {
				type = "execute",
				name = L["Clear"],
				desc = L["Clear the player and crowd control for %s."]:format(cc_symbols[i]),
				order = (9 - i) * 10 + 2,
				func = function()
					self.db.profile["RT"..i.."type"] = L["None"]
					self.db.profile["RT"..i.."name"] = ""
				end
			}
			if i ~= 1 then
				self.options.responsibilities.args["div"..i] = {
					type = "header",
					name = '',
					desc = '',
					order = (9 - i) * 10 + 3
				}
			end
		end
		targetsLoaded = true
	end
	local t = { ["SAY"] = L["Say"], ["SELF"] = L["Self"] }
	if Guilder:IsInParty() and LibGroup:GetRank() > 0 then t["PARTY"] = L["Party"] end
	if Guilder:IsInRaid() and LibGroup:GetRank() > 0 then t["RAID"] = L["Raid"] end
	if Guilder:IsOfficer() or Guilder:IsOfficerAlt() then
		t["OFFICER"] = L["Officer"]
		t["GUILD"] = L["Guild"]
	end
	self.options.announce.args.channel.values = t
end

--[[
-- Is called when the module receives a message
-- @usage internal use only
function Guilder_Crowd:CommHandler(distribution, sender, command, ...)
	-- if module is disabled, do nothing
	if not self.db.profile.enabled then
		return
	end
	-- only handle comms if they are not from yourself
	if sender == myName then return end
	local t = {}
	local n = select('#', ...)
	-- Query: Get Crowd Control
	if command == "QGCC" then
		if LibGroup:GetRank() == 2 then
			for i = 1, 8 do
				t[i] = { ["name"] = self.db.profile["RT"..i.."name"], ["type"] = self.db.profile["RT"..i.."type"] }
			end
			Guilder:SendComm(self.id, "WHISPER", sender, "RGCC", t)
		end
	end
	-- Response: Get Crowd Control
	if command == "RGCC" then
	end
end
]]

-- General functions --------------------------------------------------------

-- Reports the Crowd Control responsibilities set
-- @param distribution which channel to use. Valid options are "GUILD", "PARTY", "RAID" and "WHISPER"
-- @param target if distribution is a "WHISPER", this is the person to send the info to.
-- @usage Guilder_Crowd:AnnounceTargets("RAID")
-- Guilder_Crowd:AnnounceTargets("WHISPER", "Player")
-- @returns nil
function Guilder_Crowd:AnnounceTargets(distribution, target)
	
	-- if module is disabled, do nothing
	if not self.db.profile.enabled then
		Guilder:ShowMessage(L["%s is disabled"]:format("|cff00c0ff"..L["Crowd Control"].."|r"))
		return
	end
	local NoTargets = true
	Guilder:SendMessage(L["Crowd Control Responsibilities"], distribution, target)
	for i = 1, 8 do
		if self.db.profile["RT"..(9 - i).."type"] ~= L["None"] then
			Guilder:SendMessage("{rt"..(9 - i).."}".." - "..
				self.db.profile["RT"..(9 - i).."type"].." - "..
				self.db.profile["RT"..(9 - i).."name"], distribution, target)
			NoTargets = false
		end
	end
	if NoTargets then
		Guilder:SendMessage(L["No targets set up, expect imminent chaos!"], distribution, target)
	end
end